import os
import smtplib
from email.header import Header, make_header
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.mime.text import MIMEText

"""
Reply codes in numerical order[td]

Code	Meaning
200	(nonstandard success response, see rfc876)
211	System status, or system help reply
214	Help message
220	<domain> Service ready
221	<domain> Service closing transmission channel
250	Requested mail action okay, completed
251	User not local; will forward to <forward-path>
354	Start mail input; end with <CRLF>.<CRLF>
421	<domain> Service not available, closing transmission channel
450	Requested mail action not taken: mailbox unavailable
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
503	Bad sequence of commands
504	Command parameter not implemented
521	<domain> does not accept mail (see rfc1846)
530	Access denied (???a Sendmailism)
550	Requested action not taken: mailbox unavailable
551	User not local; please try <forward-path>
552	Requested mail action aborted: exceeded storage allocation
553	Requested action not taken: mailbox name not allowed
554	Transaction failed
Reply codes grouped by command[td]

Command	Code	Description
connect
220	<domain> Service ready
421	<domain> Service not available, closing transmission channel
HELO
250	Requested mail action okay, completed
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
504	Command parameter not implemented
521	<domain> does not accept mail [rfc1846]
421	<domain> Service not available, closing transmission channel
EHLO
250	Requested mail action okay, completed
550	Not implemented
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
504	Command parameter not implemented
421	<domain> Service not available, closing transmission channel
MAIL
250	Requested mail action okay, completed
552	Requested mail action aborted: exceeded storage allocation
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
421	<domain> Service not available, closing transmission channel
RCPT
250	Requested mail action okay, completed
251	User not local; will forward to <forward-path>
550	Requested action not taken: mailbox unavailable
551	User not local; please try <forward-path>
552	Requested mail action aborted: exceeded storage allocation
553	Requested action not taken: mailbox name not allowed
450	Requested mail action not taken: mailbox unavailable
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
503	Bad sequence of commands
521	<domain> does not accept mail [rfc1846]
421	<domain> Service not available, closing transmission channel
DATA
354	Start mail input; end with <CRLF>.<CRLF>
451	Requested action aborted: local error in processing
554	Transaction failed
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
503	Bad sequence of commands
421	<domain> Service not available, closing transmission channel
received data
250	Requested mail action okay, completed
552	Requested mail action aborted: exceeded storage allocation
554	Transaction failed
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
RSET
200	(nonstandard success response, see rfc876)
250	Requested mail action okay, completed
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
504	Command parameter not implemented
421	<domain> Service not available, closing transmission channel
SEND
250	Requested mail action okay, completed
552	Requested mail action aborted: exceeded storage allocation
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
421	<domain> Service not available, closing transmission channel
SOML
250	Requested mail action okay, completed
552	Requested mail action aborted: exceeded storage allocation
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
421	<domain> Service not available, closing transmission channel
SAML
250	Requested mail action okay, completed
552	Requested mail action aborted: exceeded storage allocation
451	Requested action aborted: local error in processing
452	Requested action not taken: insufficient system storage
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
421	<domain> Service not available, closing transmission channel
VRFY
250	Requested mail action okay, completed
251	User not local; will forward to <forward-path>
550	Requested action not taken: mailbox unavailable
551	User not local; please try <forward-path>
553	Requested action not taken: mailbox name not allowed
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
504	Command parameter not implemented
421	<domain> Service not available, closing transmission channel
EXPN
250	Requested mail action okay, completed
550	Requested action not taken: mailbox unavailable
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
504	Command parameter not implemented
421	<domain> Service not available, closing transmission channel
HELP
211	System status, or system help reply
214	Help message
500	Syntax error, command unrecognised
501	Syntax error in parameters or arguments
502	Command not implemented
504	Command parameter not implemented
421	<domain> Service not available, closing transmission channel
NOOP
200	(nonstandard success response, see rfc876)
250	Requested mail action okay, completed
500	Syntax error, command unrecognised
421	<domain> Service not available, closing transmission channel
QUIT
221	<domain> Service closing transmission channel
500	Syntax error, command unrecognised
TURN
250	Requested mail action okay, completed
502	Command not implemented
500	Syntax error, command unrecognised
503	Bad sequence of commands

"""


class Mailer:
    """
    usage:
     host = "smtp.exmail.qq.com"
    sender = "xxxx@yyy.com"
    port = 25
    password = "123456"
    receiver = ["www@qq.com"]
    cc = ["ccc@vvv.com"]

    mailer = Mailer(sender, password)
    mailer.send_mail(
    "测试邮件标题",
     "邮件内容",
      receiver,
       cc_aadrs=cc,
       attachment_files=[
            "/aaa/bb/good.pdf"
            "/aaa/bb/好样的.png"
            "/aaa/bb/不一样.mp3"
            "/aaa/bb/attach_zip.zip"
    ])
    """

    def __init__(self, sender: str, password: str, username: str = None, host: str = "smtp.exmail.qq.com",
                 port: int = 25):
        self.host = host
        self.port = port

        assert self.is_email(sender), "sender不像是邮箱"
        self.sender = sender
        self.password = password
        self.username = username if username is not None else self.sender.split('@')[0]

        self.server = smtplib.SMTP(self.host, self.port)

    @staticmethod
    def is_email(email_address: str):
        return "@" in email_address

    def login(self):
        try:
            res = self.server.login(self.sender, self.password)
        except Exception as e:
            print("登录逻辑抛出了异常")
            raise e
        return res

    def connect(self):
        try:
            res = self.server.connect(self.host, self.port)
        except Exception as e:
            print("链接邮箱服务抛出异常")
            raise e
        return res

    @staticmethod
    def attach(file):
        """
        发送邮件需要携带的附件，文件绝对路径
        :param file:
        :return:
        """
        if not os.path.exists(file):
            raise FileNotFoundError('文件{}不存在～'.format(file))

        if not os.path.isfile(file):
            raise FileNotFoundError('目标{}不是一个文件'.format(file))

        file_name = os.path.basename(file)
        with open(file, 'rb') as f:
            part = MIMEApplication(f.read())
            # 中文等特殊字符集要特殊处理
            part.add_header('Content-Disposition', 'attachment',
                            fileName="%s" % make_header([(file_name, 'UTF-8')]).encode('UTF-8'))
            return part
            # msg.attach(part)

    def send_mail(self, subject: str, content: str, to_addrs: list[str], cc_aadrs: list[str] = None,
                  subtype='mixed',
                  text_subtype='plain',
                  attachment_files: list[str] = None):

        """

        :param subject:
        :param content:
        :param to_addrs:
        :param cc_aadrs:
        :param subtype:
        :param text_subtype:
        :param attachment_files:
        :param kwargs:
        :return:
        """

        """
        邮件头信息
        """
        msgobj = MIMEMultipart(_subtype=subtype, text_subtype=text_subtype)
        msgobj['Subject'] = Header(subject, "utf-8")
        msgobj['From'] = self.sender
        msgobj['To'] = ",".join(to_addrs)
        msgobj['cc'] = ",".join(cc_aadrs) if cc_aadrs else ""

        """
        邮件内容
        """
        text = MIMEText(content, _charset='utf-8')
        # plain text
        msgobj.attach(text)

        """
        附加附件
        """
        # 如果有附件
        if attachment_files:
            for file in attachment_files:
                msgobj.attach(self.attach(file))

        """
        to list 需要包含 to+cc的总量
        """
        to_addrs = list(set(to_addrs + cc_aadrs)) if cc_aadrs else to_addrs
        # self.connect()
        self.login()
        self.server.sendmail(self.sender, to_addrs=to_addrs, msg=msgobj.as_string())


if __name__ == '__main__':
    pass



import re


def get_u_words(context: str):
    """
    从给定内容中去掉所有非母语内容，只保留母语部分
    [从给定内容中提取词]
    :param context: 需要提出词汇的内容
    :return : 返回提取后的词汇集合
    """
    context = context.strip()
    if len(context) == 0:
        return []

    context = re.sub("[^چۋېرتيۇڭوپھسداەىقكلزشغۈبنمئلاۆجخگفژٴحعۉۅ]", " ", context)  # 非母语替换成空白
    return context.split()


if __name__ == '__main__':
    pass
    target_contents = "你好么,hello -world, ياخشىمۇ سىز"
    print("target_contents,txt_to_words", target_contents, get_u_words(target_contents))



"""
处理文件句柄
"""


def get_lines(file_path: str, mode="r", encoding="utf-8"):
    with open(file_path, mode=mode, encoding=encoding) as f:
        file_lines = [line.strip() for line in f.readlines()]
    return file_lines


def write_lines(file_path, lines=None, mode="w", encoding="utf-8", newline_separator="\n"):
    if lines is None:
        lines = []
    if lines is None or len(lines) == 0:
        return False
    with open(file_path, mode=mode, encoding=encoding) as target:
        for line in lines:
            line = line.strip()
            target.write(line)
            target.write(newline_separator)


class File:
    def __init__(self, file_path):
        pass


if __name__ == '__main__':
    pass



import praatio.tgio as tgio
from praatio.tgio import IntervalTier, Textgrid, Interval
from decimal import Decimal


class SIntervalHandler:
    """
    textgrid中 item 里的intervals
    """

    def __init__(self, interval: Interval, index=0):
        """
        :param  interval:
        """
        self.index = index
        self.interval = interval

        self.start = interval.start
        self.end = interval.end
        self.label = interval.label

    def get_time(self, decimal_type=True):
        """
        Decimal(f"{tg_text.start:.06f}")
        :param decimal_type:
        :param decimal_leng:
        :return:
        """
        if decimal_type:
            return [Decimal(f"{self.start:.06f}"), Decimal(f"{self.end:.06f}")]

        return [self.start, self.end]

    def get_diff_time(self, other_interval):
        """
        返回两个 intervals的起始和结尾点的误差
        :param SIntervalHandler other_interval:
        :return:
        """
        text_start, text_end = self.get_time()
        spkr_start, spkr_end = other_interval.get_time()

        return [abs(text_start - spkr_start), abs(text_end - spkr_end)]

    def is_time_equal(self, other_interval):
        """
        比较两个interval 的起始和结尾点是否一致
        :param SIntervalHandler other_interval:
        :return:
        """
        diff = list(set(self.get_diff_time(other_interval)))

        if len(diff) > 1:
            return False
        if Decimal(diff[0]) > 0:
            return False

        return True

    def __repr__(self):
        return f"intervals[{self.index}], min:{self.start}, max:{self.end},text:{self.label}"

    def __str__(self):
        return f"intervals[{self.index}], min:{self.start}, max:{self.end},text:{self.label}"


class IntervalErrorBean:
    def __init__(self, text_interval: SIntervalHandler, speaker_interval: SIntervalHandler, error_message: list[str]):
        self.text_interval = text_interval
        self.speaker_interval = speaker_interval
        self.error_message = error_message
        self.time_diff = [str(item) for item in self.text_interval.get_diff_time(self.speaker_interval)]

    def __repr__(self):
        return "\n".join([" ".join(self.error_message),
                          str(self.text_interval), str(self.speaker_interval),
                          "起始对，结尾对误差", " ".join(self.time_diff), "---" * 5])

    def __str__(self):
        return "\n".join([" ".join(self.error_message),
                          str(self.text_interval), str(self.speaker_interval),
                          "起始对，结尾对误差", " ".join(self.time_diff), "---" * 5])


class STierHandler:
    """
    textgrid 中的item
    """

    def __init__(self, tier: list[SIntervalHandler]):
        self.tier = tier

    def entrylist(self):
        return [(item.start, item.end, item.label) for item in self.tier]

    def get_tier_diffs(self, other_tier):
        """
        返回所有text item和speaker item不一致的条和他们的错误信息
        :param STierHandler other_tier:
        :return:
        """
        assert len(self.tier) > 0 and len(other_tier.tier), "tier数量不等于2或者存在空tier,比较条件不成立"

        tier_diffs = []
        pre_end = 0
        index = 0
        for tg_text, tg_speaker in zip(self.tier, other_tier.tier):
            index = index + 1

            text_begin, text_end = tg_text.get_time()
            spkr_begin, spkr_end = tg_speaker.get_time()

            err_message = []
            # 判断时间是否对应
            if not tg_text.is_time_equal(tg_speaker):
                err_message.append("时间不对应")
            # 判断时间是否连续
            if spkr_begin != pre_end:
                err_message.append("时间不连续")

            pre_end = text_end

            if len(err_message) > 0:
                tier_diffs.append(IntervalErrorBean(tg_text, tg_speaker, err_message))
        return tier_diffs

    def align_time(self, need_aligned_tier):
        """
        按照自身的每个 entry 的起始和结束时间来对齐传递进来的tier的起始和结束时间
        :param STierHandler other_tier:
        :return:
        """
        assert len(self.tier) > 0 and len(need_aligned_tier.tier), "tier数量不等于2或者存在空tier,无法对齐时间"
        newEntrylist = []
        for selfz, target in zip(self.tier, need_aligned_tier.tier):  # type:SIntervalHandler
            newEntrylist.append((selfz.start, selfz.end, target.label))
        return newEntrylist


class STextGridHandler:
    """
    textgrid文件句柄
    """

    def __init__(self, path):
        self.path = path
        # type:Textgrid
        try:
            self.tg = tgio.openTextgrid(self.path, readRaw=True)
        except AssertionError as e:
            assert len(e.args) != 0, "两个tier的名字相同导致程序无法正确获取段列表"
        # assert tgio.openTextgrid(self.path, readRaw=True), "分析库报错，两段tier名字相同导致的问题"

    def tierNameList(self):
        """
        返回两个tier的名字列表
        :return:
        """
        return self.tg.tierNameList

    def entryList(self):
        """
        返回转换SIntervalHandler后的[item0,item1]集合
        :return:
        """
        entryLists = []
        for item in self.tg.tierDict:
            iter = self.tg.tierDict.get(item)  # type:IntervalTier
            slist = []
            for index in range(len(iter.entryList)):
                slist.append(SIntervalHandler(iter.entryList[index], index))
            entryLists.append(slist)

        return entryLists

    def tier(self, tier_name):
        assert tier_name in self.tg.tierDict, "tier name 不存在"

        return self.tg.tierDict[tier_name].entryList

    def tier_diffs(self):
        assert len(self.entryList()) >= 2, "只有一个tier，少于2的tier无法产出比较值"

        enlist = self.entryList()
        one, two = enlist[0], enlist[1]
        # assert len(one) == len(two), "两个声道段数不一致"

        return STierHandler(one).get_tier_diffs(STierHandler(two))

    def time_diffs(self):
        """
        返回错误误差集
        :return:
        """
        diffs = []
        res = self.tier_diffs()
        if res:
            for item in res:
                diffs.extend(item.time_diff)

        return sorted(diffs)

    def align_text_speaker_time(self):
        """
        item1 和item2的每个item起始点和结束点自动对齐,参考item1的时间对齐item2
        :return:
        """
        tg = self
        one, two = tg.entryList()[:2]
        oneTier = STierHandler(one)
        twoTier = STierHandler(two)
        newEntrylist = oneTier.align_time(twoTier)
        tg.tg.tierDict[tg.tg.tierNameList[0]].entryList = oneTier.entrylist()
        tg.tg.tierDict[tg.tg.tierNameList[1]].entryList = newEntrylist
        # out_path = os.path.join(os.path.dirname(file), "time_aligned", os.path.basename(file))
        # if os.path.exists(os.path.dirname(out_path)) is not True:
        #     os.makedirs(os.path.dirname(out_path), exist_ok=True)
        # tg.tg.save(out_path)

    def save(self, save_path):
        self.tg.save(save_path, useShortForm=False)

    def __str__(self):
        return self.path

#
# def get_error_details(allTextGrids, out_path="所有问题textgrid详情"):
#     """
#     生成所有文件的异常详情记录
#     :param allTextGrids:
#     :return:
#     """
#     if not os.path.exists(out_path):
#         os.makedirs(out_path, exist_ok=True)
#     ff = ""
#     try:
#         for file in allTextGrids:
#             error_msg = [file]
#             ff = file
#
#             tg = STextGridHandler(file)
#             diffs = tg.time_diffs()
#             if len(diffs) > 0:
#                 error_msg.extend([str(item) for item in tg.tier_diffs()])
#             if len(error_msg) == 1:  # 只有文件名时不计为有问题
#                 continue
#             write_lines(os.path.join(out_path, os.path.basename(file) + ".txt"), error_msg)
#     except AssertionError as e:
#         pass
#         print("异常", ff, e)
#
#
# def get_diff_tierlen_files(allTextGrids):
#     """
#     两段数量不相等
#     :return:
#     """
#     files = []
#     try:
#         for file in allTextGrids:
#
#             [one, two] = (STextGridHandler(file).entryList())[:2]
#             if len(one) != len(two):
#                 print(len(one), len(two))
#                 print(file, "两段数量不相等")
#                 files.append(file)
#
#     except AssertionError as e:
#         pass
#     return set(files)
#
#
# def get_diff_times_gt_1(allTextGrids, out_path=False):
#     """
#     误差大于1秒
#     :return:
#     """
#     files = []
#     try:
#         for file in allTextGrids:
#             diffs = STextGridHandler(file).tier_diffs()
#             if diffs:
#                 for item in diffs:
#                     gt_1 = list(filter(lambda x: Decimal(x) > 1, item.time_diff))
#                     if len(gt_1) > 0:
#                         print(file, "误差大于1秒")
#                         files.append(file)
#                         if out_path:
#                             os.makedirs(out_path, exist_ok=True)
#                             shutil.copy(file, out_path)
#                         break
#
#
#     except AssertionError as e:
#         pass
#     return set(files)
#
#
# def get_breaking_points(one, two):
#     break_pointes_list = []
#     for index, target_interval in enumerate(one):  # type:int,SIntervalHandler
#         if index > len(two) - 1:  # 索引不匹配的情况
#             break
#         other_interval = two[index]  # type:SIntervalHandler
#         # 起始点不相等，结束点相同情况
#         if target_interval.is_time_equal(other_interval) is False:
#             start_diff, end_diff = target_interval.get_diff_time(other_interval)
#             if (start_diff == 0) and (end_diff != 0):
#                 one_ends = [item.end for item in one]
#                 two_ends = [item.end for item in two]
#
#                 res1 = other_interval.end in one_ends  #
#                 res2 = target_interval.end in two_ends
#                 if res1:
#                     # print(f"item2 的 interval {other_interval.index} 在 item1 上变成了多段")
#                     break_pointes_list.append(f"item2 的 interval {other_interval.index} 在 item1 上变成了多段")
#                 if res2:
#                     # print(f"item1 的 interval {other_interval.index} 在 item2 上变成了多段")
#                     break_pointes_list.append(f"item1 的 interval {other_interval.index} 在 item2 上变成了多段")
#     return break_pointes_list
#
#
# def find_2text_1speaker(allTextGrids, out_path=False):
#     """
#     找item1的 一段编程了两段的情况
#     :param allTextGrids:
#     :return:
#     """
#     try:
#         for file in allTextGrids:
#             tg = STextGridHandler(file)
#             all_entry_list = tg.entryList()
#             if len(all_entry_list) < 2:
#                 print("这文件没有 >=2个item，无法做比较")
#                 continue
#             [one, two] = all_entry_list[:2]  # type:list[list[SIntervalHandler]]
#             res = get_breaking_points(one, two)
#             if len(res) > 0:
#                 print(f"文件{os.path.basename(file)} 中出现以下问题:")
#                 print("\n".join(res))
#                 if out_path:
#                     os.makedirs(out_path, exist_ok=True)
#                     shutil.copy(file, out_path)
#     except Exception as e:
#         pass
#         print("异常", e)
#
#
# def get_all_diff_times(allTextGrids, out_path):
#     try:
#         diffs = []
#
#         for file in allTextGrids:
#             tg = STextGridHandler(file)
#             res = tg.time_diffs()
#             for item in res:
#                 if Decimal(item) > 0.06:
#                     print(os.path.basename(file), item)
#                     if out_path:
#                         os.makedirs(out_path, exist_ok=True)
#                         shutil.copy(file, out_path)
#                         break
#                 diffs.extend(res)
#
#         result = sorted(list(Counter(diffs).items()), key=lambda x: x[0], reverse=True)
#         write_lines("difftime-list.txt", ["  ".join([item[0], str(item[1])]) for item in result])
#     except AssertionError as e:
#         print("AssertionError", e)
#     except Exception as e:
#         pass
#         print("异常", e)
#
#
# def align_time(allTextGrids, out_path):
#     """
#     按照item1 对齐item2的起始和结束时间
#     :param allTextGrids:
#     :return:
#     """
#     try:
#         result_output_path = out_path
#         for file in allTextGrids:
#
#             tg = STextGridHandler(file)
#             tg.align_text_speaker_time()
#
#             out_path = os.path.join(result_output_path, *(file.split("/")[-2:-1]))
#             if os.path.exists(out_path) is not True:
#                 os.makedirs(out_path, exist_ok=True)
#
#             tg.save(os.path.join(out_path, os.path.basename(file)))
#
#
#     except AssertionError as e:
#         print("AssertionError", e)
#     except Exception as e:
#         pass
#         print("异常", e)
#
#
# def makeup_path(allTextGrids):
#     for file in allTextGrids:
#         path = os.path.splitext(os.path.basename(file))[0]
#         real_path = os.path.join(os.path.dirname(file), str(int(path)))
#         os.makedirs(real_path)
#         shutil.move(file, real_path)
#         # print(path,int(path))
#
#
# if __name__ == '__main__':
#     pass
#
#     """
#     pip install praatio
#     pip install shirkhan
#     """
#     # allTextGrids = [""]
#     # allTextGrids = glob.glob("/Users/babbage/Desktop/中文客服联想返工/data_2000小时/29761_data/**/*.TextGrid")
#     # allTextGrids = glob.glob("/Users/babbage/Desktop/中文客服联想返工/data_2000小时/所有问题修正后的textgrids/**/*.TextGrid")
#     # allTextGrids = glob.glob("/Users/babbage/Desktop/中文客服联想返工/返工200小时详情/中文客服200小时数据/**/*.TextGrid")
#     # allTextGrids = glob.glob("/Users/babbage/Desktop/中文客服联想返工/返工200小时详情/200小时修复后的textgrids/**/*.TextGrid")
#
#     # print("分析的textgrid总数量", len(allTextGrids))
#
#     # 生成目录下所有文件的错误详情到给定目录下
#     # 1.
#     # get_error_details(allTextGrids, "all_2380_error_detail_final")
#     # 2.获取段数不一致的textgrid文件列表
#     # files = get_diff_tierlen_files(allTextGrids)
#     # 3. 获取 误差大于1的文件列表
#     # files = get_diff_times_gt_1(allTextGrids,"时间误差超过1秒textgrids")
#     # 4. 找出item1分割成2个的段位,如果outpath 给了数字，程序将问题textgrdi复制到这个目录下
#     # find_2text_1speaker(allTextGrids, out_path="中文客服200小时段被分割的textgrids/")
#     # 5. 拿出所有误差,写入 difftime_list.txt文件中
#     # get_all_diff_times(allTextGrids, "存在误差大于0.06的textgrids/")
#     # 6. 时间对齐
#     # align_time(allTextGrids, "/Users/babbage/Desktop/中文客服联想返工/返工200小时详情/200小时修复后的textgrids/")
#     # 7. 恢复路径
#     # files=glob.glob("/Users/babbage/Desktop/中文客服联想返工/返工200小时详情/问题和过程/修改完-存在误差大于0.06的textgrids/*.TextGrid")
#     # makeup_path(files)



import xlrd
from xlrd.sheet import Sheet


class xls:
    def __init__(self, file, *args, **kwargs):
        pass
        # 判断文件是否存在
        # 判断文件后缀是否对的上
        # 判断是否可以打开
        # type:xlrd.Book
        self.book = xlrd.open_workbook(file, *args, **kwargs)

    @property
    def active(self):
        return self.sheets()[0]

    def sheet_names(self):
        """
        返回sheet names列表
        :return:
        """
        return self.book.sheet_names()

    def sheets(self):
        """
        返回sheets对象列表
        :return:
        """
        return self.book.sheets()

    def sheet_by_name(self, sheet_name: str):
        """
        通过名字获取sheet对象
        :param sheet_name:
        :return:
        """
        return self.book.sheet_by_name(sheet_name)

    def sheet_by_index(self, sheet_index):
        """
        通过下标获取sheet对象
        :param sheet_index:
        :return:
        """
        return self.book.sheet_by_index(sheet_index)

    def rows(self, sheet_name):
        """
        返回指定sheet名下的所有行
        :param sheet_name:
        :return:
        """
        return self.sheet_by_name(sheet_name).get_rows()

    def rows_counts(self, sheet_name: str):
        """
        返回指定sheet有多少行
        :param sheet_name:
        :return:
        """
        return self.sheet_by_name(sheet_name).nrows

    def row_values(self, sheet_name: str, rowx, start_colx=0, end_colx=None):
        """
        返回指定区域的行列表
        :param sheet_name:
        :param rowx:
        :param start_colx:
        :param end_colx:
        :return:
        """
        return self.sheet_by_name(sheet_name).row_values(rowx, start_colx, end_colx)

    def row_types(self, sheet_name: str, rowx: int, start_colx=0, end_colx=None):
        """
        返回指定行的 字段类型的数值列表
        :param sheet_name:
        :param rowx:
        :param start_colx:
        :param end_colx:
        :return:
        """
        return self.sheet_by_name(sheet_name).row_types(rowx, start_colx, end_colx)

    def cols_counts(self, sheet_name: str):
        """
        返回指定sheet有多少列
        :param sheet_name:
        :return:
        """
        return self.sheet_by_name(sheet_name).ncols

    def col_values(self, sheet_name: str, colx, start_rowx=0, end_rowx=None):
        """
        返回指定区域的列列表
        :param sheet_name:
        :param colx:
        :param start_rowx:
        :param end_rowx:
        :return:
        """
        return self.sheet_by_name(sheet_name).col_values(colx, start_rowx, end_rowx)

    def col_types(self, sheet_name: str, colx: int, start_rowx=0, end_rowx=None):
        """
        返回指定列的 字段类型数值列表
        :param sheet_name:
        :param colx:
        :param start_rowx:
        :param end_rowx:
        :return:
        """
        return self.sheet_by_name(sheet_name).col_types(colx, start_rowx, end_rowx)

    @staticmethod
    def col_name(colx):
        # 通过给定的行和列下标推算并返回列名字
        return xlrd.colname(colx)

    def cell(self, sheet_name: str, rowx: int, colx: int):
        """
       通过行和列下标获取 cell 对象
        :param sheet_name:
        :param rowx:
        :param colx:
        :return:
        """
        return self.sheet_by_name(sheet_name).cell(rowx, colx)

    def cell_value(self, sheet_name: str, rowx: int, colx: int):
        """
        返回指定sheet名下的指定下标的cell的值
        :param sheet_name:
        :param rowx:
        :param colx:
        :return:
        """
        return self.sheet_by_name(sheet_name).cell_value(rowx, colx)

    @staticmethod
    def cell_name(rowx, colx):
        """
        通过给定的行和列下标推算并返回单元格名字
        :param rowx:
        :param colx:
        :return:
        """
        return xlrd.cellname(rowx, colx)

    @staticmethod
    def load_workbook(file, *args, **kwargs):
        """
        打开excel 返回实例
        :param file:
        :param args:
        :param kwargs:
        :return:
        """
        return xls(file, *args, **kwargs)



import wave
from pydub import AudioSegment

'''
分析音频常用linux工具库:
sox
soxi 查看音频详情 metadata

开源pyton 库
pydub
'''


def open(wav_path):
    with wave.open(wav_path) as w:  # type:wave.Wave_read
        return w


def get_duration(wav_path):
    f = open(wav_path)
    frames = f.getnframes()
    rate = f.getframerate()
    duration = float(frames) / float(rate)
    return duration


def get_channels(wav_path):
    f = open(wav_path)
    return f.getnchannels()


def get_framerate(wav_path):
    f = open(wav_path)
    return f.getframerate()


def get_sampwidth(wav_path):
    f = open(wav_path)
    return f.getsampwidth()


def cut(wav_path, start_time, end_time, out_path):
    """
    给定的wav按照 start_time 和 end_time 切割并存放在out_path中
    :param wav_path: 音频路径
    :param start_time: 起始点
    :param end_time:结束点
    :param out_path: 保存的文件路径 xx.wav
    :return:
    """
    song = AudioSegment.from_wav(wav_path)
    t1 = start_time * 1000
    t2 = end_time * 1000
    newAudio = song[t1:t2]
    newAudio.export(out_path, format="wav")
    return True


if __name__ == '__main__':
    pass



import re

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("shirkhan/__init__.py", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
    name="shirkhan",
    version=version,
    author="shirkhan",
    author_email="uybabbage@hotmail.com",
    description="shirkhan tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ishirkhan",
    packages=find_packages(),
    install_requires=[
        "pycryptodome",
        "xlrd",
        "openpyxl",
        "pydub",
        "praatio",
    ],

    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
